Attribute VB_Name = "VBVSL_TwoAddons_Main"
Option Explicit

'---------------------------------------------------------------------
' VBVSL_TwoAddons_Main.BAS module
' Implementation for methods deferred from VBVSL.cls
'----------------------------------------------------------------------

'======================================================================
' Some handy VSL Globals
' Set up as a user-defined type so that IntelliSense will work :-)
'======================================================================

'-------------------------
Public Enum VSL_Consts
'-------------------------
   AddOnMin = 1
   Addon_Args = AddOnMin
   Addon_Events
   AddOnMax = Addon_Events
End Enum

'---------------------------
Public Type VSL_Globals
'---------------------------
  VisioApp As Visio.Application
  VVA_Callback As VBVSL_Types.IVVA_Callback
  
  AddOns(AddOnMax) As IVBVSL_AddOn
End Type

Public glo As VSL_Globals

'==========================================================
' VSL-level methods
'==========================================================

'------------------------------------
Public Sub VBVSL_Initialize()
'------------------------------------
' Perform any early initialization, but note later opportunity to
' do so when Visio specifically requests to prepare for calls
' in VBVSL_Load and/or VBVSL_

End Sub

'------------------------------------
Public Sub VBVSL_Terminate()
'------------------------------------
' Perform any local cleanup, destroy objects etc. if not already done
' in VBVSL_Unload. At this stage it's too late to talk to Visio.

  ForgetAddOnObjects
End Sub

'------------------------------------------------------------------------------------
Public Sub VBVSL_VSLInfo(Descr As String)
'------------------------------------------------------------------------------------
  Descr = "VBVSL_Sample VSL and AddOns"
End Sub

'------------------------------------------------------------------------------------
Public Function VBVSL_Load(ByVal InterfaceVersion As Long) As VBVSL_Types.Tvbvsl_ResultCodes
'------------------------------------------------------------------------------------
' Prepare for RUN calls. Create any objects needed in order to act intelligent.
Dim AddOnIx As Integer

  On Error Resume Next
  VBVSL_Load = vbvsl_L2V_LOADFAILED
    
  AddOnObjectsCheck ' create AddOns if not already
  
  For AddOnIx = AddOnMin To AddOnMax
    If glo.AddOns(AddOnIx).Load = vbvsl_SUCCESS Then VBVSL_Load = vbvsl_SUCCESS
    ' in this implementation, we'll respond FAILURE only if no addons succeeded
  Next AddOnIx
End Function

'------------------------------------------------------------------------------------
Public Function VBVSL_Unload() As VBVSL_Types.Tvbvsl_ResultCodes
'------------------------------------------------------------------------------------
' Perform any local cleanup, destroy objects etc.
Dim AddOnIx As Integer
  On Error Resume Next
  
  For AddOnIx = AddOnMin To AddOnMax
    glo.AddOns(AddOnIx).Unload
  Next AddOnIx

  '--------------------------------
  ' ... and finally...
  '--------------------------------
  ForgetAddOnObjects
End Function

'------------------------------------------------------------------------------------
Public Function VBVSL_EnumAddOns(ByVal SessionId As Long) As VBVSL_Types.Tvbvsl_ResultCodes
'------------------------------------------------------------------------------------
Dim AddOnIx As Integer
Dim Attribs As VBVSL_Types.Tvbvsl_AddOnInfo_Attribs
Dim EnableMask As VBVSL_Types.Tvbvsl_AddOnInfo_EnableMask
Dim InvokeOnMask As VBVSL_Types.Tvbvsl_AddOnInfo_InvokeOnMask
Dim NotifyOnMask As VBVSL_Types.Tvbvsl_AddOnInfo_NotifyOnMask
Dim Name As String

  AddOnObjectsCheck ' Create the AddOns if not already
  
  For AddOnIx = AddOnMin To AddOnMax
    glo.AddOns(AddOnIx).AddOnInfo Attribs, EnableMask, InvokeOnMask, NotifyOnMask, Name
    glo.VVA_Callback.RegisterAddOn SessionId, AddOnIx, Attribs, EnableMask, InvokeOnMask, NotifyOnMask, Name
  Next AddOnIx
  
  VBVSL_EnumAddOns = vbvsl_SUCCESS
End Function

'==========================================================
' AddOn-level methods
'==========================================================

'------------------------------------------------------------------------------------
Public Function VBVSL_IsAddOnEnabled(ByVal AddOnIx As Long, _
             ByVal SessionId As Long) As VBVSL_Types.Tvbvsl_ResultCodes
'------------------------------------------------------------------------------------
  On Error Resume Next
  If Not WithinBounds(AddOnIx, AddOnMin, AddOnMax) Then GoTo skipit
  
  VBVSL_IsAddOnEnabled = 0        ' 0 = Not Enabled! (ie: doesn't mean success)
  AddOnObjectsCheck
  
  If Not (glo.AddOns(AddOnIx) Is Nothing) Then
    VBVSL_IsAddOnEnabled = glo.AddOns(AddOnIx).IsAddOnEnabled
  End If
skipit:
End Function

'------------------------------------------------------------------------------------
Public Function VBVSL_Run(ByVal AddOnIx As Long, ByVal SessionId As Long, _
                  ByVal Args As String) As VBVSL_Types.Tvbvsl_ResultCodes
'------------------------------------------------------------------------------------
  On Error Resume Next
  If Not WithinBounds(AddOnIx, AddOnMin, AddOnMax) Then GoTo skipit

  AddOnObjectsCheck
  
  If Not (glo.AddOns(AddOnIx) Is Nothing) Then
    VBVSL_Run = glo.AddOns(AddOnIx).Run(SessionId, Args)
  End If
skipit:
End Function

'------------------------------------------------------------------------------------
Public Sub VBVSL_KillSession(ByVal SessionId As Long)
'------------------------------------------------------------------------------------
' End a session for a prior RUN that hasn't ended yet
Dim AddOnIx As Integer
  On Error Resume Next
  For AddOnIx = AddOnMin To AddOnMax
    If Not (glo.AddOns(AddOnIx) Is Nothing) Then
      glo.AddOns(AddOnIx).KillSession SessionId ' ask addon to end session if SessionId matches
    End If
  Next AddOnIx
End Sub

'------------------------------------------------------------------------------------
Public Function VBVSL_Test(ByVal I As Long) As Long
'------------------------------------------------------------------------------------
' Test call FROM VVA
  VBVSL_Test = I * 3   ' a simple response
End Function

'====================================================================================
' Some services to other modules in this project
'====================================================================================

'------------------------------------------------------------------------------------
Public Function SendTest(ByVal I As Long) As Long
'------------------------------------------------------------------------------------
' Test call TO VVA
  If Not (glo.VVA_Callback Is Nothing) Then
    SendTest = glo.VVA_Callback.Test(I)
  End If
End Function

'------------------------------------------------------------------------------------
Public Sub SendEndSession(ByVal SessionId As Long)
'------------------------------------------------------------------------------------
  On Error Resume Next
  If Not (glo.VVA_Callback Is Nothing) Then
    glo.VVA_Callback.SendEndSession SessionId
  End If
End Sub

'------------------------------------------------------------------------------------
Public Sub SendTraceMessage(Msg As String)
'------------------------------------------------------------------------------------
  On Error Resume Next
  If Not (glo.VVA_Callback Is Nothing) Then
    glo.VVA_Callback.AdapterMessage vbvsl_AdapterMessage_AddMessage, Msg
  End If
End Sub

'====================================================================================
' Some generic local supporting methods
'====================================================================================

'-------------------------------------------
Private Sub AddOnObjectsCheck()
'-------------------------------------------
  On Error Resume Next
  If glo.AddOns(1) Is Nothing Then
    Set glo.AddOns(Addon_Args) = New VBVSL_TwoAddons_Args
    Set glo.AddOns(Addon_Events) = New VBVSL_TwoAddons_Evts
  End If
End Sub

'-------------------------------------------
Private Sub ForgetAddOnObjects()
'-------------------------------------------
Dim AddOnIx As Integer
  On Error Resume Next
  For AddOnIx = AddOnMin To AddOnMax
    Set glo.AddOns(AddOnIx) = Nothing
  Next AddOnIx
End Sub

